package main

import androidx.compose.foundation.layout.*
import androidx.compose.material.*
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.filled.ArrowBack
import androidx.compose.material.icons.filled.Check
import androidx.compose.runtime.Composable
import androidx.compose.runtime.MutableState
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.unit.dp
import kotlinx.serialization.encodeToString
import kotlinx.serialization.json.Json
import java.io.File
import java.io.IOException

@OptIn(ExperimentalMaterialApi::class)
@Composable
fun formPage(glossary: Glossary, onCancelClick: () -> Unit) {
    // State to track whether to show the snackbar
    val requiredFieldsSnackbarVisibleState = remember { mutableStateOf(false) }
    val alreadyExistSnackbarVisibleState = remember { mutableStateOf(false) }

    val name = remember { mutableStateOf("") }
    val description = remember { mutableStateOf("") }
    val mainContext = remember { mutableStateOf("") }
    val secondaryContext = remember { mutableStateOf("") }
    val relatedTo = remember { mutableStateOf("") }
    val synonymous = remember { mutableStateOf("") }
    val antonym = remember { mutableStateOf("") }

    val openDialog = remember { mutableStateOf(false) }
    if (openDialog.value) {
        AlertDialog(
            onDismissRequest = {
                openDialog.value = false
            },
            title = {
                Text(text = "Confirmation d'ajout de mot")
            },
            text = {
                Text("Le mot '${name.value}' a bien été ajouté au glossaire.")
            },
            confirmButton = {
                Button(
                    onClick = {
                        openDialog.value = false
                        resetFields(name, description, mainContext, secondaryContext, relatedTo, synonymous, antonym)

                    },
                    colors = ButtonDefaults.buttonColors(
                        backgroundColor = customRedColor,
                        contentColor = Color.White
                    )
                ) {
                    Text("Fermer")

                }
            }
        )
    }

    MaterialTheme {
        Column(
            modifier = Modifier.padding(16.dp),
            verticalArrangement = Arrangement.spacedBy(8.dp),
            horizontalAlignment = Alignment.CenterHorizontally
        ) {
            Text(text = "Nouveau mot", style = MaterialTheme.typography.h5)

            TextField(
                value = name.value,
                onValueChange = { name.value = it },
                label = { Text("Mot *") },
                colors = TextFieldDefaults.textFieldColors(
                    focusedIndicatorColor = customRedColor,
                    unfocusedIndicatorColor = Color.Gray,
                    focusedLabelColor = customRedColor
                )
            )

            TextField(
                value = description.value,
                onValueChange = { description.value = it },
                label = { Text("Description") },
                colors = TextFieldDefaults.textFieldColors(
                    focusedIndicatorColor = customRedColor,
                    unfocusedIndicatorColor = Color.Gray,
                    focusedLabelColor = customRedColor
                )
            )

            TextField(
                value = mainContext.value,
                onValueChange = { mainContext.value = it },
                label = { Text("Contexte principal *") },
                colors = TextFieldDefaults.textFieldColors(
                    focusedIndicatorColor = customRedColor,
                    unfocusedIndicatorColor = Color.Gray,
                    focusedLabelColor = customRedColor
                )
            )

            TextField(
                value = secondaryContext.value,
                onValueChange = { secondaryContext.value = it },
                label = { Text("Contexte 2") },
                colors = TextFieldDefaults.textFieldColors(
                    focusedIndicatorColor = customRedColor,
                    unfocusedIndicatorColor = Color.Gray,
                    focusedLabelColor = customRedColor
                )
            )

            TextField(
                value = relatedTo.value,
                onValueChange = { relatedTo.value = it },
                label = { Text("Lié à") },
                colors = TextFieldDefaults.textFieldColors(
                    focusedIndicatorColor = customRedColor,
                    unfocusedIndicatorColor = Color.Gray,
                    focusedLabelColor = customRedColor
                )
            )

            TextField(
                value = synonymous.value,
                onValueChange = { synonymous.value = it },
                label = { Text("Synonyme") },
                colors = TextFieldDefaults.textFieldColors(
                    focusedIndicatorColor = customRedColor,
                    unfocusedIndicatorColor = Color.Gray,
                    focusedLabelColor = customRedColor
                )
            )

            TextField(
                value = antonym.value,
                onValueChange = { antonym.value = it },
                label = { Text("Antonyme") },
                colors = TextFieldDefaults.textFieldColors(
                    focusedIndicatorColor = customRedColor,
                    unfocusedIndicatorColor = Color.Gray,
                    focusedLabelColor = customRedColor
                )
            )

            Row(
                modifier = Modifier.fillMaxWidth(),
                horizontalArrangement = Arrangement.SpaceEvenly
            ) {
                Button(
                    onClick = onCancelClick,
                    colors = ButtonDefaults.buttonColors(
                        backgroundColor = customRedColor,
                        contentColor = Color.White
                    )
                ) {
                    Icon(imageVector = Icons.Default.ArrowBack, contentDescription = null)
                    Text("Retour")
                }
                Button(
                    onClick = {
                        // Validate the fields
                        if (name.value.isBlank() || mainContext.value.isBlank()) {
                            // Show the snackbar when validation fails
                            requiredFieldsSnackbarVisibleState.value = true
                            return@Button
                        }

                        val wordsList = loadDatasFromFile(glossary.jsonFilePath).toMutableList()

                        // Transform name to lowercase to avoid duplicates
                        name.value = name.value.lowercase()

                        // Verify if the word already exists in the glossary
                        if (wordsList.any { it.name.equals(name.value, ignoreCase = true) }) {
                            println("Le mot '${name.value}' existe déjà dans le glossaire. Ajout annulé.")
                            alreadyExistSnackbarVisibleState.value = true
                            return@Button
                        }

                        val newWord = Word(
                            name = name.value,
                            description = description.value,
                            mainContext = mainContext.value,
                            secondaryContext = secondaryContext.value,
                            relatedTo = relatedTo.value,
                            synonymous = synonymous.value,
                            antonym = antonym.value
                        )

                        addWordToGlossary(glossary.jsonFilePath, newWord)
                        openDialog.value = true

                    },
                    colors = ButtonDefaults.buttonColors(
                        backgroundColor = customRedColor,
                        contentColor = Color.White
                    )
                ) {
                    Icon(imageVector = Icons.Default.Check, contentDescription = null)
                    Text("Valider")
                }
            }

            // Show the snackbar when the state is true
            if (requiredFieldsSnackbarVisibleState.value) {
                Snackbar(
                    modifier = Modifier.padding(16.dp),
                    action = {
                        Button(
                            onClick = { requiredFieldsSnackbarVisibleState.value = false },
                            colors = ButtonDefaults.buttonColors(
                                backgroundColor = customRedColor,
                                contentColor = Color.White
                            )
                            ) {
                            Text("OK")
                        }
                    }
                ) {
                    Text("Les champs 'Mot' et 'Contexte principal' sont obligatoires.")
                }
            }
            if (alreadyExistSnackbarVisibleState.value) {
                Snackbar(
                    modifier = Modifier.padding(16.dp),
                    action = {
                        Button(
                            onClick = { alreadyExistSnackbarVisibleState.value = false },
                            colors = ButtonDefaults.buttonColors(
                                backgroundColor = customRedColor,
                                contentColor = Color.White
                            )
                            ) {
                            Text("OK")
                        }
                    }
                ) {
                    Text("Le mot '${name.value}' existe déjà dans le glossaire. Ajout annulé.")
                }
            }
        }
    }
}

fun resetFields(
    name: MutableState<String>,
    description: MutableState<String>,
    mainContext: MutableState<String>,
    secondaryContext: MutableState<String>,
    relatedTo: MutableState<String>,
    synonymous: MutableState<String>,
    antonym: MutableState<String>
) {
    name.value = ""
    description.value = ""
    mainContext.value = ""
    secondaryContext.value = ""
    relatedTo.value = ""
    synonymous.value = ""
    antonym.value = ""
}

fun addWordToGlossary(filePath: String, newWord: Word) {
    val wordsList = loadDatasFromFile(filePath).toMutableList()

    // Verify if the word already exists in the glossary
    if (wordsList.any { it.name.equals(newWord.name, ignoreCase = true) }) {
        println("Le mot '${newWord.name}' existe déjà dans le glossaire. Ajout annulé.")
        return
    }

    // Add the new word to the glossary
    wordsList.add(newWord)
    saveDatasInFile(filePath, wordsList)

    println("Mot ajouté avec succès : ${newWord.name}")
}

fun saveDatasInFile(filePath: String, listeWords: List<Word>) {
    val json = Json { prettyPrint = true }
    try {
        val content = json.encodeToString(listeWords)
        File(filePath).writeText(content)
    } catch (e: IOException) {
        e.printStackTrace()
    }
}

fun loadDatasFromFile(filePath: String): List<Word> {
    // If file is empty, return empty list
    if (!File(filePath).exists() || File(filePath).length() == 0L) {
        return emptyList()
    }
    return try {
        val content = File(filePath).readText()
        Json.decodeFromString<List<Word>>(content)
    } catch (e: IOException) {
        e.printStackTrace()
        emptyList() // Handle the exception as needed
    }
}